<?php // $Id: listing_premium.admin.inc,v 1.0 2011/03/17 09:00:32 newzeal Exp $
function listing_premium_settings_form() {
  $form['listing_premium_node_view_alert'] = array(
    '#type'          => 'checkbox',
    '#title'         => t('Node View Alert'),
    '#default_value' => variable_get('listing_premium_node_view_alert', 1),
    '#description'   => t("Display a link for the owner of a listing node on the view page to the premium listings application page if the node is not already upgraded."),
  );

  $form['#validate'][] = 'listing_premium_settings_validate';
  return system_settings_form($form);
}
function listing_premium_settings_validate($form, $form_state) {
  if (!is_numeric($form_state['values']['listing_premium_levels'])) form_set_error('listing_premium_levels', t('The number of levels has to be numeric'));
}

function listing_premium_types_admin() {
  $output = l('Add a Premium Listing Type', 'admin/listing/premium/types/add');
  $result = db_query("SELECT * FROM {listing_premium_types}");
  $rows = array();
  $header = array('Name', 'Description', 'Operation');
  while ($row = db_fetch_object($result)) {
    $rows[$row->lpid]['name'] = $row->name;
    $rows[$row->lpid]['description'] = $row->description;
    $rows[$row->lpid]['edit'] = l('Edit', 'admin/listing/premium/types/'.$row->lpid.'/edit');
  }
  $output .= theme('table', $header, $rows);
  return $output;
}
function listing_premium_type_form($form_state, $listing_type=NULL) {
  $form['name'] = array(
      '#type'          => 'textfield',
      '#title'         => t('Name'),
      '#default_value' => isset($listing_type->name) ? $listing_type->name : '',
      '#description'   => t("The name of this premium listing level."),
  );
  $form['description'] = array(
      '#type'          => 'textfield',
      '#title'         => t('Description'),
      '#default_value' => isset($listing_type->description) ? $listing_type->description : '',
      '#description'   => t("The description for this premium listing level."),
  );
  $types = uc_product_types();
  $placeholders = array();
  foreach ($types as $type) $placeholders[] = "'%s'";
  $options = array();
  $result = db_query("SELECT nid, title FROM {node} WHERE type IN (".implode(",", $placeholders).")", $types);
  while ($row = db_fetch_object($result)) {
	  $options[$row->nid] = $row->title;
  }
  $form['product_nid'] = array(
      '#type'          => 'select',
	  '#options' 	   => $options,
      '#title'         => t('Product'),
      '#default_value' => isset($listing_type->product_nid) ? $listing_type->product_nid : '',
      '#description'   => t("The product to associate when purchasing this premium listing level."),
  );
  $options = array();
  foreach (content_fields() as $field) {
    //drupal_set_message(print_r($field,1));
	$options[$field['field_name']] = $field['widget']['label']. ' (' .$field['type_name']. ')';
  }
  $form['data'] = array(
      '#type'          => 'select',
	  '#options' 	   => $options,
	  '#multiple'	   => TRUE,
      '#title'         => t('Fields'),
      '#default_value' => isset($listing_type->data) ? unserialize($listing_type->data) : array(),
      '#description'   => t("Whichever fields you select here will not display unless the node has been upgraded using this listing type or by another listing type."),
  );
  $form['duration_units'] = array(
	  '#prefix' => '<div>'. t('The length of time which the upgrade applies for.  Note that this will not be used if a listing is mapped to a product through uc_node_checkout and uses attributes.').'</div>',
	  '#type' => 'textfield',
	  '#size' => 4,
	  '#title' => t('Number of Units'),
	  '#default_value' => isset($listing_type->duration_units) ? $listing_type->duration_units : 1,
	  '#description' => t('A zero value will disable this feature and node will publish immediately on purchase.'),
  );
  $form['duration_period'] = array(
	  '#type' => 'select',
	  '#title' => t('Period'),
	  '#options' => uc_node_checkout_scheduler_duration_options(),
	  '#default_value' => isset($listing_type->duration_period) ? $listing_type->duration_period : '',
  );
  $form['lpid'] = array(
      '#type'          => 'hidden',
      '#value'         => isset($listing_type->lpid) ? $listing_type->lpid : 0,
  );
  $form['save'] = array(
      '#type'          => 'submit',
      '#value'         => t('Save'),
  );
  $form['delete'] = array(
      '#type'          => 'submit',
      '#value'         => t('Delete'),
  );
  return $form;
}
function listing_premium_type_form_submit($form, $form_state) {
  //drupal_set_message(print_r($form_state['values'],1));
  if ($form_state['values']['op'] == "Save") {
    if ($form_state['values']['lpid'] > 0) {
      db_query("UPDATE {listing_premium_types} SET name='%s', description='%s', product_nid=%d, duration_units=%d, duration_period='%s', data='%s' WHERE lpid=%d", $form_state['values']['name'], $form_state['values']['description'], $form_state['values']['product_nid'], $form_state['values']['duration_units'], $form_state['values']['duration_period'], serialize($form_state['values']['data']), $form_state['values']['lpid']);
	  $lpid = $form_state['values']['lpid'];
    }
    else {
      db_query("INSERT INTO {listing_premium_types} (name, description, product_nid, duration_units, duration_period, data) VALUES ('%s', '%s', %d, '%s')", $form_state['values']['name'], $form_state['values']['description'], $form_state['values']['product_nid'], $form_state['values']['duration_units'], $form_state['values']['duration_period'], serialize($form_state['values']['data']));
	  $lpid = db_last_insert_id('listing_premium_types', 'lpid');
    }
    db_query("DELETE FROM {listing_premium_fields} WHERE lpid=%d", $lpid);
    foreach ($form_state['values']['data'] as $field) {
      db_query("INSERT INTO {listing_premium_fields} (field_name, lpid, active) VALUES ('%s', %d, 1)", $field, $lpid);
    }
  }
  drupal_goto('admin/listing/premium/types');
}